#include <stdio.h>
#include <math.h>
#include "mha_plugin.hh"
#include "mha_events.h"


class cfg_t {
public:
    cfg_t(unsigned int,unsigned int);
    unsigned int channel;
};

class example6_t : public MHAPlugin::plugin_t<cfg_t> {
public:
	// Declare Constructor
    example6_t(algo_comm_t iac, const std::string & configured_name);
    // Declare Process function
    mha_wave_t* process(mha_wave_t*);
    // Declare Prepare function
    void prepare(mhaconfig_t&);
    // Declare AC Processing function
    void calculate_mean_value();
    // Declare wave structure used to store the AC variable
    mha_wave_t store_ac_single;

private:
    void update_cfg();
    MHAParser::int_t channel_no;
    MHAEvents::patchbay_t<example6_t> patchbay;
    MHAParser::string_t lsl_time_series;
    float mean_value;
};

MHAPLUGIN_CALLBACKS(steffencov,example6_t,wave,wave)

cfg_t::cfg_t(unsigned int ichannel,
             unsigned int numchannels)
    : channel(ichannel)
{
    if( channel >= numchannels )
        throw MHA_Error(__FILE__,__LINE__,
                        "Invalid channel number %u (only %u channels configured).",
                        channel,numchannels);
}


example6_t::example6_t(algo_comm_t iac, const std::string &)
    : MHAPlugin::plugin_t<cfg_t>("Example rms level meter plugin",iac),
      /* initialzing variable 'channel_no' with MHAParser::int_t(char* name, .... ) */
      channel_no("channel in which the RMS level is measured","0","[0,["),
      lsl_time_series("Name of LSL stream to be processed"," ")


{
    /* Register variables to the configuration parser: */
    insert_item("channel",&channel_no);
    insert_item("ac_name",&lsl_time_series);

    patchbay.connect(&channel_no.writeaccess,this,&example6_t::update_cfg);

}

void example6_t::calculate_mean_value() {
	// Index for going through data chunk
	unsigned int fr;
	// Get EEG/LSL data
    store_ac_single = MHA_AC::get_var_waveform(ac,lsl_time_series.data);
    // Set mean value to 0
    mean_value = 0.0;
    // Sum up all the values of time series
    for(fr = 0; fr < store_ac_single.num_frames*store_ac_single.num_channels; fr++){
        mean_value += store_ac_single.buf[fr];
    }
    // Normalize by divding by number of elements
    mean_value = mean_value/(store_ac_single.num_frames*store_ac_single.num_channels);
    // Insert mean value as new AC variable
    ac.insert_var_float( ac.handle, "Mean_value", &mean_value );
}


mha_wave_t* example6_t::process(mha_wave_t* wave)
{
	calculate_mean_value();
	poll_config();
    	return wave;
}
/* *******************Prepare******************* */
void example6_t::prepare(mhaconfig_t& tfcfg)
{
calculate_mean_value();
    if( tfcfg.domain != MHA_WAVEFORM )
        throw MHA_Error(__FILE__,__LINE__,"Example6: Only waveform processing is supported.");
    /* remember the transform configuration (i.e. channel numbers): */
    tftype = tfcfg;
    /* make sure that a valid runtime configuration exists: */
    update_cfg();
}
/* *******************Update_cfg******************* */
void example6_t::update_cfg()
{
    if( tftype.channels )
        push_config(new cfg_t(channel_no.data,tftype.channels));
}

MHAPLUGIN_DOCUMENTATION\
(steffencov,
 "example feature-extraction algorithm-communication",
 "This plugin calculates the RMS level of a given channel of the input signal,"
 " working in the time domain.\n"
 "The channel number is made accessible to the configuration structure and\n"
 "the result is stored into a algorithm communication variable (AC variable).")
